"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Generic event/listener system
 */
var GunEvent = /** @class */ (function () {
    function GunEvent(name) {
        if (name === void 0) { name = 'GunEvent'; }
        this.name = name;
        this._listeners = [];
        this.listenerCount = this.listenerCount.bind(this);
        this.on = this.on.bind(this);
        this.off = this.off.bind(this);
        this.trigger = this.trigger.bind(this);
    }
    /**
     * @returns number of currently subscribed listeners
     */
    GunEvent.prototype.listenerCount = function () {
        return this._listeners.length;
    };
    /**
     * Register a listener on this event
     *
     * @param cb the callback to subscribe
     */
    GunEvent.prototype.on = function (cb) {
        if (this._listeners.indexOf(cb) !== -1)
            return;
        this._listeners.push(cb);
        return this;
    };
    /**
     * Unregister a listener on this event
     * @param cb the callback to unsubscribe
     */
    GunEvent.prototype.off = function (cb) {
        var idx = this._listeners.indexOf(cb);
        if (idx !== -1)
            this._listeners.splice(idx, 1);
        return this;
    };
    /**
     * Unregister all listeners on this event
     */
    GunEvent.prototype.reset = function () {
        this._listeners = [];
        return this;
    };
    /**
     * Trigger this event
     */
    GunEvent.prototype.trigger = function (a, b, c) {
        this._listeners.forEach(function (cb) { return cb(a, b, c); });
        return this;
    };
    return GunEvent;
}());
exports.GunEvent = GunEvent;
//# sourceMappingURL=GunEvent.js.map