"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var GunEvent_1 = require("../ControlFlow/GunEvent");
var GunGraphUtils_1 = require("./GunGraphUtils");
/**
 * Query state around a single node in the graph
 */
var GunGraphNode = /** @class */ (function () {
    function GunGraphNode(graph, soul, updateGraph) {
        this._data = new GunEvent_1.GunEvent("<GunGraphNode " + soul + ">");
        this._graph = graph;
        this._updateGraph = updateGraph;
        this.soul = soul;
    }
    GunGraphNode.prototype.listenerCount = function () {
        return this._data.listenerCount();
    };
    GunGraphNode.prototype.get = function (cb) {
        if (cb)
            this.on(cb);
        this._ask();
        return this;
    };
    GunGraphNode.prototype.receive = function (data) {
        this._data.trigger(data, this.soul);
        return this;
    };
    GunGraphNode.prototype.on = function (cb) {
        this._data.on(cb);
        return this;
    };
    GunGraphNode.prototype.off = function (cb) {
        if (cb) {
            this._data.off(cb);
        }
        else {
            this._data.reset();
        }
        if (this._endCurQuery && !this._data.listenerCount()) {
            this._endCurQuery();
            this._endCurQuery = undefined;
        }
        return this;
    };
    GunGraphNode.prototype._ask = function () {
        var _this = this;
        if (this._endCurQuery)
            return;
        var endFns = [];
        var msgId = GunGraphUtils_1.generateMessageId();
        this._graph.eachConnector(function (connector) {
            endFns.push(connector.get({
                msgId: msgId,
                soul: _this.soul,
                cb: _this._onDirectQueryReply.bind(_this)
            }));
        });
        this._endCurQuery = function () { return endFns.map(function (fn) { return fn(); }); };
        return this;
    };
    GunGraphNode.prototype._onDirectQueryReply = function (msg) {
        var _a;
        if ('put' in msg && !msg.put) {
            this._updateGraph((_a = {},
                _a[this.soul] = undefined,
                _a), msg['@']);
        }
    };
    return GunGraphNode;
}());
exports.GunGraphNode = GunGraphNode;
//# sourceMappingURL=GunGraphNode.js.map