"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArrays = (this && this.__spreadArrays) || function () {
    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;
    for (var r = Array(s), k = 0, i = 0; i < il; i++)
        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
            r[k] = a[j];
    return r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var EMPTY = {};
function generateMessageId() {
    return Math.random()
        .toString(36)
        .slice(2);
}
exports.generateMessageId = generateMessageId;
function addMissingState(graphData) {
    var updatedGraphData = __assign({}, graphData);
    var now = new Date().getTime();
    for (var soul in graphData) {
        var node = graphData[soul];
        if (!node)
            continue;
        var meta = (node._ = node._ || {});
        meta['#'] = soul;
        var state = (meta['>'] = meta['>'] || {});
        for (var key in node) {
            if (key === '_')
                continue;
            state[key] = state[key] || now;
        }
        updatedGraphData[soul] = node;
    }
    return updatedGraphData;
}
exports.addMissingState = addMissingState;
var DEFAULT_OPTS = {
    futureGrace: 10 * 60 * 1000,
    Lexical: JSON.stringify // what gun.js uses
};
function diffGunCRDT(updatedGraph, existingGraph, opts) {
    if (opts === void 0) { opts = DEFAULT_OPTS; }
    var _a = opts || EMPTY, _b = _a.machineState, machineState = _b === void 0 ? new Date().getTime() : _b, _c = _a.futureGrace, futureGrace = _c === void 0 ? DEFAULT_OPTS.futureGrace : _c, _d = _a.Lexical, Lexical = _d === void 0 ? DEFAULT_OPTS.Lexical : _d;
    var maxState = machineState + futureGrace; // eslint-disable-line
    var allUpdates = {};
    for (var soul in updatedGraph) {
        var existing = existingGraph[soul];
        var updated = updatedGraph[soul];
        var existingState = (existing && existing._ && existing._['>']) || EMPTY;
        var updatedState = (updated && updated._ && updated._['>']) || EMPTY;
        if (!updated) {
            if (!(soul in existingGraph))
                allUpdates[soul] = updated;
            continue;
        }
        var hasUpdates = false;
        var updates = {
            _: {
                '#': soul,
                '>': {}
            }
        };
        for (var key in updatedState) {
            var existingKeyState = existingState[key];
            var updatedKeyState = updatedState[key];
            if (updatedKeyState > maxState || !updatedKeyState)
                continue;
            if (existingKeyState && existingKeyState >= updatedKeyState)
                continue;
            if (existingKeyState === updatedKeyState) {
                var existingVal = (existing && existing[key]) || undefined;
                var updatedVal = updated[key];
                // This is based on Gun's logic
                if (Lexical(updatedVal) <= Lexical(existingVal))
                    continue;
            }
            updates[key] = updated[key];
            updates._['>'][key] = updatedKeyState;
            hasUpdates = true;
        }
        if (hasUpdates) {
            allUpdates[soul] = updates;
        }
    }
    return Object.keys(allUpdates) ? allUpdates : undefined;
}
exports.diffGunCRDT = diffGunCRDT;
function diffSets(initial, updated) {
    return [
        updated.filter(function (key) { return initial.indexOf(key) === -1; }),
        initial.filter(function (key) { return updated.indexOf(key) === -1; })
    ];
}
exports.diffSets = diffSets;
function mergeGunNodes(existing, updates, mut) {
    if (mut === void 0) { mut = 'immutable'; }
    if (!existing)
        return updates;
    if (!updates)
        return existing;
    var existingMeta = existing._;
    var existingState = existingMeta['>'];
    var updatedMeta = updates._;
    var updatedState = updatedMeta['>'];
    if (mut === 'mutable') {
        for (var key in updatedState) {
            existing[key] = updates[key];
            existingState[key] = updatedState[key];
        }
        return existing;
    }
    return __assign(__assign(__assign({}, existing), updates), { _: {
            '#': existingMeta['#'],
            '>': __assign(__assign({}, existingMeta['>']), updates._['>'])
        } });
}
exports.mergeGunNodes = mergeGunNodes;
function mergeGraph(existing, diff, mut) {
    if (mut === void 0) { mut = 'immutable'; }
    var result = mut ? existing : __assign({}, existing);
    for (var soul in diff) {
        result[soul] = mergeGunNodes(existing[soul], diff[soul], mut);
    }
    return result;
}
exports.mergeGraph = mergeGraph;
function nodeToGraph(node) {
    var _a;
    var modified = __assign({}, node);
    var nodes = {};
    var nodeSoul = node && node._ && node._['#'];
    for (var key in node) {
        if (key === '_')
            continue;
        var val = node[key];
        if (typeof val !== 'object')
            continue;
        var soul = val && val._ && val._['#'];
        if (soul) {
            var edge = { '#': soul };
            modified[key] = edge;
            var graph = addMissingState(nodeToGraph(val));
            var diff_1 = diffGunCRDT(graph, nodes);
            nodes = diff_1 ? mergeGraph(nodes, diff_1) : nodes;
        }
    }
    var raw = (_a = {}, _a[nodeSoul] = modified, _a);
    var withMissingState = addMissingState(raw);
    var diff = diffGunCRDT(withMissingState, nodes);
    nodes = diff ? mergeGraph(nodes, diff) : nodes;
    return nodes;
}
exports.nodeToGraph = nodeToGraph;
function flattenGraphData(data) {
    var graphs = [];
    var flatGraph = {};
    for (var soul in data) {
        var node = data[soul];
        if (node)
            graphs.push(nodeToGraph(node));
    }
    for (var i = 0; i < graphs.length; i++) {
        var diff = diffGunCRDT(graphs[i], flatGraph);
        flatGraph = diff ? mergeGraph(flatGraph, diff) : flatGraph;
    }
    return flatGraph;
}
exports.flattenGraphData = flattenGraphData;
function getPathData(keys, graph) {
    var lastKey = keys[keys.length - 1];
    if (keys.length === 1) {
        return {
            souls: keys,
            value: graph[lastKey],
            complete: lastKey in graph
        };
    }
    var _a = getPathData(keys.slice(0, keys.length - 1), graph), parentValue = _a.value, souls = _a.souls;
    if (typeof parentValue !== 'object') {
        return {
            souls: souls,
            value: undefined,
            complete: typeof parentValue !== 'undefined'
        };
    }
    var value = parentValue[lastKey];
    if (!value) {
        return {
            souls: souls,
            value: value,
            complete: true
        };
    }
    var edgeSoul = value['#'];
    if (edgeSoul) {
        return {
            souls: __spreadArrays(souls, [edgeSoul]),
            value: graph[edgeSoul],
            complete: edgeSoul in graph
        };
    }
    return {
        souls: souls,
        value: value,
        complete: true
    };
}
exports.getPathData = getPathData;
//# sourceMappingURL=GunGraphUtils.js.map