"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var GunEvent_1 = require("../ControlFlow/GunEvent");
var GunProcessQueue_1 = require("../ControlFlow/GunProcessQueue");
var GunGraphConnector = /** @class */ (function () {
    function GunGraphConnector(name) {
        if (name === void 0) { name = 'GunGraphConnector'; }
        this.isConnected = false;
        this.name = name;
        this.inputQueue = new GunProcessQueue_1.GunProcessQueue(name + ".inputQueue");
        this.outputQueue = new GunProcessQueue_1.GunProcessQueue(name + ".outputQueue");
        this.events = {
            graphData: new GunEvent_1.GunEvent(name + ".events.graphData"),
            receiveMessage: new GunEvent_1.GunEvent(name + ".events.receiveMessage"),
            connection: new GunEvent_1.GunEvent(name + ".events.connection")
        };
        this.__onConnectedChange = this.__onConnectedChange.bind(this);
        this.events.connection.on(this.__onConnectedChange);
    }
    GunGraphConnector.prototype.waitForConnection = function () {
        var _this = this;
        if (this.isConnected)
            return Promise.resolve();
        return new Promise(function (ok) {
            var onConnected = function (connected) {
                if (!connected)
                    return;
                ok();
                _this.events.connection.on(onConnected);
            };
            _this.events.connection.on(onConnected);
        });
    };
    /**
     * Send graph data for one or more nodes
     *
     * @returns A function to be called to clean up callback listeners
     */
    GunGraphConnector.prototype.put = function (_a) {
        var graph = _a.graph, _b = _a.msgId, msgId = _b === void 0 ? '' : _b, _c = _a.replyTo, replyTo = _c === void 0 ? '' : _c, cb = _a.cb;
        return function () { };
    };
    /**
     * Request data for a given soul
     *
     * @returns A function to be called to clean up callback listeners
     */
    GunGraphConnector.prototype.get = function (_a) {
        var soul = _a.soul, cb = _a.cb, _b = _a.msgId, msgId = _b === void 0 ? '' : _b, _c = _a.key // TODO
        , key = _c === void 0 ? '' : _c // TODO
        ;
        return function () { };
    };
    /**
     * Queues outgoing messages for sending
     *
     * @param msgs The Gun wire protocol messages to enqueue
     */
    GunGraphConnector.prototype.send = function (msgs) {
        this.outputQueue.enqueueMany(msgs);
        if (this.isConnected)
            this.outputQueue.process();
    };
    /**
     * Queue incoming messages for processing
     *
     * @param msgs
     */
    GunGraphConnector.prototype.ingest = function (msgs) {
        this.inputQueue.enqueueMany(msgs).process();
    };
    GunGraphConnector.prototype.__onConnectedChange = function (connected) {
        if (connected) {
            this.isConnected = true;
            this.outputQueue.process();
        }
        else {
            this.isConnected = false;
        }
    };
    return GunGraphConnector;
}());
exports.GunGraphConnector = GunGraphConnector;
//# sourceMappingURL=GunGraphConnector.js.map