import { ChainGun } from './ChainGun';
export declare class ChainGunLink {
    key: string;
    soul?: string;
    private _updateEvent;
    private _chain;
    private _parent?;
    private _endQuery?;
    private _lastValue;
    private _hasReceived;
    constructor(chain: ChainGun, key: string, parent?: ChainGunLink);
    getPath(): string[];
    /**
     * Traverse a location in the graph
     *
     * @param key Key to read data from
     * @param cb
     * @returns New chain context corresponding to given key
     */
    get(key: string, cb?: GunMsgCb): ChainGunLink;
    /**
     * Move up to the parent context on the chain.
     *
     * Every time a new chain is created, a reference to the old context is kept to go back to.
     *
     * @param amount The number of times you want to go back up the chain. {-1} or {Infinity} will take you to the root.
     * @returns a parent chain context
     */
    back(amount?: number): ChainGunLink | ChainGun;
    /**
     * Save data into gun, syncing it with your connected peers.
     *
     * You do not need to re-save the entire object every time, gun will automatically
     * merge your data into what already exists as a "partial" update.
     *
     * @param value the data to save
     * @param cb an optional callback, invoked on each acknowledgment
     * @returns same chain context
     */
    put(value: GunValue, cb?: GunMsgCb): this;
    /**
     * Add a unique item to an unordered list.
     *
     * Works like a mathematical set, where each item in the list is unique.
     * If the item is added twice, it will be merged.
     * This means only objects, for now, are supported.
     *
     * @param data should be a gun reference or an object
     * @param cb The callback is invoked exactly the same as .put
     * @returns chain context for added object
     */
    set(data: any, cb?: GunMsgCb): void;
    /**
     * Register a callback for when it appears a record does not exist
     *
     * If you need to know whether a property or key exists, you can check with .not.
     * It will consult the connected peers and invoke the callback if there's reasonable certainty that none of them have the data available.
     *
     * @param cb If there's reason to believe the data doesn't exist, the callback will be invoked. This can be used as a check to prevent implicitly writing data
     * @returns same chain context
     */
    not(cb: (key: string) => void): this;
    /**
     * Change the configuration of this chain link
     *
     * @param options
     * @returns same chain context
     */
    opt(options: GunChainOptions): this;
    /**
     * Get the current data without subscribing to updates. Or undefined if it cannot be found.
     *
     * @param cb The data is the value for that chain at that given point in time. And the key is the last property name or ID of the node.
     * @returns same chain context
     */
    once(cb: GunOnCb): this;
    /**
     * Subscribe to updates and changes on a node or property in realtime.
     *
     * Triggered once initially and whenever the property or node you're focused on changes,
     * Since gun streams data, the callback will probably be called multiple times as new chunk comes in.
     *
     * To remove a listener call .off() on the same property or node.
     *
     * @param cb The callback is immediately fired with the data as it is at that point in time.
     * @returns same chain context
     */
    on(cb: GunOnCb): this;
    /**
     * Unsubscribe one or all listeners subscribed with on
     *
     * @returns same chain context
     */
    off(cb?: GunOnCb): this;
    promise(opts?: {
        timeout: number;
    }): Promise<GunValue>;
    then(fn?: (val: GunValue) => any): Promise<any>;
    /**
     * Iterates over each property and item on a node, passing it down the chain
     *
     * Not yet supported
     *
     * Behaves like a forEach on your data.
     * It also subscribes to every item as well and listens for newly inserted items.
     *
     * @returns a new chain context holding many chains simultaneously.
     */
    map(): ChainGunLink;
    /**
     * No plans to support this
     */
    path(path: string): ChainGunLink;
    /**
     * No plans to support this
     */
    open(cb: Function): void;
    /**
     * No plans to support this
     */
    load(cb: Function): void;
    /**
     * No plans to support this
     */
    bye(): void;
    /**
     * No plans to support this
     */
    later(): void;
    /**
     * No plans to support this
     */
    unset(node: GunNode): void;
    private _onQueryResponse;
}
